/********************************************************************

	main.c

	Alarm Clock

	CPU:                dsPIC30F3012(use +3.3V)
	Real Time Clock:    EPSONTOYOCOM RTC-8564NB
	LCD:                I2C Small LCD

	CPU Clock(+3.3V)
	     internal FRC, PLLx8, Fcy=64MHz(7.37Mx8), Tcy=16MHz(14.74M)

	CPU I/O Pin ASSIGN

	 I2C for RTC-8564NB/Small LCD
	  INT1/RD0  (10) Alarm or Interval Timer interrupt in
	  SCL/RB4   (11) I2C Clock   Pullup 2K in RTC-8564
	  SDA/RB5   (12) I2C Data IO Pullup 2K in RTC-8564

	 Push Button (switch)
	  RB0        (2)  SET
	  RB1        (3)  UP
	  RB2        (4)  DOWN
	  INT0/RB6  (16)  Button Wakeup Interrupt

	 Buzzer
	  OC2/RB7   (15)  Buzzer

	 UART to PC
	  U1ATX/RC13 (8)  UART TX
	  U1ARX/RC14 (9)  UART RX


	                 MCR#(1) (18)AVDD -------- +3V3
	Button(SET)  ---  RB0(2) (17)AVSS -------- GND
	Button(UP)   ---  RB1(3) (16)RB6/INT0 ---- Button Interrupt
	Button(DOWN) ---  RB2(4) (15)OC2/RB7 ----- Buzzer 
	                  RB3(5) (14)VDD --------- +3V3
	                 OSC1(6) (13)VSS --------- GND
	            OSC2/RC15(7) (12)PGC/SDA/RB5 - I2C Data
	UART TX -- U1ATX/RC13(8) (11)PGD/SCL/RB4 - I2C Clock
	UART RX -- U1ARX/RC14(9) (10)INT1/RD0 ---- Alarm or Interval Timer interrupt


	@
	tE̐ݒ@
	@SET@Ł@tE̐ݒJn
	   NAAAjAAAb@̏ɐݒB
	@@@NJnASET@Ŏ̍ڂ֐iށB
	@@@b̐ݒ̎@SET@ŏI
	   UP,DOWNŊeڂ̒l+1,-1oB

	A[ij̐ݒ@
	@SET+UP ŐݒJn
	   ȀɐݒBSETŕݒɐiށB
	@@@̐ݒ̎@SET@ŏI
	   UP,DOWNŊeڂ̒l+1,-1oB

	A[̃IIt
	  UP   ŃA[I
	  DOWN ŃA[It 

	L[̔F
	SET,UP,DOWN@̃L[́AĂ痣ɔFB
	SET+UP ́ASET ėO UP āAɗB
	L[邱Ƃɂ郊s[g͋@\͖B

*********************************************************************/

#include <p30f3012.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "rtc8564.h"
#include "lcd.h"
#include "key.h"


/*=======================================================================
	Device Configuration
=======================================================================*/

//_FOSC(CSW_FSCM_OFF & FRC_PLL16) // no clock change, internal RC osc 7.37M x PLL16
_FOSC(CSW_FSCM_OFF & FRC_PLL8) // no clock change, internal RC osc 7.37M x PLL8
_FWDT(WDT_OFF) // watch dog timer off
_FBORPOR(PBOR_OFF & MCLR_EN)
_FGS(CODE_PROT_OFF)
_FICD(ICS_PGD)


/*=======================================================================
	Definition
=======================================================================*/

#define	ITEM_YEAR	6
#define	ITEM_MONTH	5
#define	ITEM_WDAY	4
#define	ITEM_DAY	3
#define	ITEM_HOUR	2
#define	ITEM_MINUTE	1
#define	ITEM_SECOND	0

#define	ALARM_WDAY		3
#define	ALARM_DAY		2
#define	ALARM_HOUR		1
#define	ALARM_MINUTE	0


/*=======================================================================
	Global Variable
=======================================================================*/

// name of weekday
const char	*wday_name[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", };

// time set operation control data
const struct	{
	int		idx;
	char	*format;
	int		max;
	int		min;
} item[] = {
	{ ITEM_YEAR,   " Year   -> %04d ", 99, 0, },
	{ ITEM_MONTH,  " Month  ->   %02d ", 12, 1, },
	{ ITEM_DAY,    " Day    ->   %02d ", 31, 1, }, 
	{ ITEM_WDAY,   " Wday   ->  %3s ",  6,  0, },
	{ ITEM_HOUR,   " Hour   ->   %02d ", 23, 0, },
	{ ITEM_MINUTE, " Minute ->   %02d ", 59, 0, },
	{ ITEM_SECOND, " Second ->   %02d ", 59, 0, },
};

// Alarm On/Off set flag
static	int	AlarmSet;


/*======================================================================
	function prototype
======================================================================*/

// function prototype
void	io_initialize(void);
void	alarm_clock(void);
void	alarm_bell(void);
void	key_operation(void);
void	clock_set(void);
void	alarm_set(void);
void	alarm_on(void);
void	alarm_off(void);
void	display_date_time(unsigned char data[]);
void	display_alarm_time(unsigned char data[]);


/*======================================================================
	main
======================================================================*/

int	main(void)
{
	// system initialize
	io_initialize();
	rs_initialize();
	lcd_initialize();
	rtcInit();


//rs_putstr("Start Alarm Clock! \r\n");

	// alarm off
	AlarmSet = 0;

	// rtcc interval timer set
	rtcSetIntervalTimer(60);

	// go alarm clock
	alarm_clock();

	return 0;
}


/*---------------------------------------------------------------------------------------------------------------------------------
	initialize io, i2c, interrupt etc.
---------------------------------------------------------------------------------------------------------------------------------*/

void	io_initialize(void)
{
	//--- setting interrupt ---------------------------------------
	INTCON1 = 0;			// NSTDIS=0
	INTCON2bits.INT0EP = 1;	// INT0 interrupt detect falling edge.
	INTCON2bits.INT1EP = 1;	// INT1 interrupt detect falling edge. 
	IEC0bits.INT0IE = 0;	// disable INT0 interrupt
	IEC1bits.INT1IE = 0;	// disable INT1 interrupt
	IFS0bits.INT0IF = 0;	// clar INT0 interrupt flag
	IFS1bits.INT1IF = 0;	// clear INT1 interrupt flag

	//--- setting I/O port ----------------------------------------
	ADPCFG = 0b1111111111111111;// Analog pin all disable
	TRISB = 0b0000000001000111;	// RB0,RB1,RB2,RB6 are input, other output
	TRISC = 0b0100000000000000;	// RC14 is input, other output.
	TRISD = 0b0000000000000001;	// RD0 is input, other output
	LATB = 0;	// output pin to low level
	LATC = 0;	// output pin to low level
	LATD = 0;	// output pin to low level

	//--- setting I2C ---------------------------------------------
	I2CCON = 0;		// disable I2C, set parameter default value.
	I2CBRG = 93;	// Fsc=400KHZ, Fcy=7.37x8=58.96MHz,
					// (58.96MHZ/400KHZ-58.96MHZ/1.111111MHZ)-1=147.4-53.06-1=94.34-1=93
	I2CADD = 0x57;	// slave module own address, not use. 0x57 is same 0xAE.

	//--- setting OC2 ---------------------------------------------
	// set Timer2 for PWM Pulse generate 
	T2CON = 0;				// stop timer
	T2CONbits.TSIDL = 1;	// disable in idle mode
	T2CONbits.TCKPS = 0;	// prescale 1:1
	TMR2 = 0;
	//PR2 = 14739;			// Pulse Frequency, 14.74Mhz/14740=1KHz
	PR2 = 2947;				// Pulse Frequency, 14.74MHz/2948=5KHz

	// PWM 5KHz continues pulse
	OC2CONbits.OCM = 0;		// disable OC module
	OC2CONbits.OCSIDL = 1;	// disable idle mode
	OC2CONbits.OCFLT = 0;	// don't use Fault pin
	OC2CONbits.OCTSEL = 0;	// use TMR2
	//OC2R = 7369;			// initial duty 50% (7370/14740)
	//OC2RS = 7369;			// duty 50% (7370/14740)
	OC2R = 1474;			// initial duty 50% (1474/2948)
	OC2RS = 1474;			// duty 50% (1474/2948)

}


/*=====================================================================
	main Operation
=====================================================================*/

void	alarm_clock(void)
{
	unsigned char flag;
	unsigned char tm[7];

	// initial display
	rtcReadNowTime(tm);
	display_date_time(tm);
	rtcReadAlarmTime(tm);
	display_alarm_time(tm);
	delay_ms(500);

	for(;;){

		// clear interrupt cause flag
		key_interrupt = 0;
		rtc_interrupt = 0;


//rs_putstr("To Sleep!\r\n");
//delay_ms(100);

		// enable Key interrupt, Alarm or Interval timer interrupt 
		IFS0bits.INT0IF = 0;	// clear INT0 interrupt flag
		IFS1bits.INT1IF = 0;	// clear INT1 interrupt flag
		IEC0bits.INT0IE = 1;	// enable INT0(Key) interrupt
		IEC1bits.INT1IE = 1;	// enable INT1(Alarm) interrupt

		// sleep
		asm("PWRSAV	#0");	// to sleep

		IEC0bits.INT0IE = 0;	// disable INT0(Key) interrupt
		IEC1bits.INT1IE = 0;	// disable INT1(Alarm) interrupt

//rs_putstr("Wake up from Sleep!\r\n");

		// display now date & time
		rtcReadNowTime(tm);
		display_date_time(tm);

		// check wake up cause	
		if(key_interrupt){	// INT0 Key interrupt

//rs_putstr("key interrupt!\r\n");

			// disable interrupt
			//IEC0bits.INT0IE = 0;	// disable INT0(Key) interrupt
			//IEC1bits.INT1IE = 0;	// disable INT1(Alarm) interrupt
			//IFS1bits.INT1IF = 0;	// clear INT1 interrupt flag

			// clar INT0 flag
			IFS0bits.INT0IF = 0;	// clear INT0 interrupt flag


			// key operation, clock set, alarm on/off etc
			key_operation();

		}else{	// INT1 Alarm, Timer interrupt

//rs_putstr("Clock Timer interrupt!\r\n");

			// disable interrupt
			//IEC1bits.INT1IE = 0;	// disable INT1(Alarm) interrupt
			//IEC0bits.INT0IE = 0;	// disable INT0(Key) interrupt
			//IFS0bits.INT0IF = 0;	// clear INT0 interrupt flag
			
			// clear INT1 flag
			IFS1bits.INT1IF = 0;	// clear INT1 interrupt flag

			// display now date & time
			//rtcReadNowTime(tm);
			//display_date_time(tm);

			// check Alarm flag
			flag = rtcGetIF();

//myprintf("alarm flag=%02x\r\n", flag);

			if(AlarmSet && (flag & alarm_if)){
				// bell on
				alarm_bell();

				rtcReadNowTime(tm);
				display_date_time(tm);
			}

			// clear Interval Alarm  Flag
			rtcClearIF();
		}

	}	// loop forever

}


/*===================================================================
	alarm bell sound out
===================================================================*/

void	alarm_bell(void)
{

	// playing untill any key push
	while(getkey() == KEY_NONE){

		// sounder sound play by 5KHz 
		OC2CONbits.OCM = 0b110;
		T2CONbits.TON = 1;
		delay_ms(200);
		// sound stop
		T2CONbits.TON = 0;
		OC2CONbits.OCM = 0b000;
		delay_ms(200);

	}

}


/*===================================================================
	Key Operation
===================================================================*/

void	key_operation(void)
{

	switch(getkey()){
	case KEY_SET:
		clock_set();
		// rtcc interval timer set
		rtcSetIntervalTimer(60);
		break;

	case KEY_ALRM_SET:
		alarm_set();
		// not break;
		// Auto set "alarm on" after alarm time set operation.
	case KEY_UP:
		alarm_on();
		break;

	case KEY_DOWN:
		alarm_off();
		break;

	case 0x07:	// bell sound test
		alarm_bell();
		break;

	default:
		break;
	}
}


/*=============================================================
	Clock Set Operation

	date & time Register offset 2
		data[0] seconds
		data[1] minutes
		data[2] hours
		data[3] days
		data[4] weekdays
		data[5] months
		data[6] year

	alarm time Register offset 9 
		data[0] minute alarm
		data[1] hour alarm
		data[2] day alarm
		data[3] weekday alarm

========================================================*/

void	clock_set(void)
{
	char	data[7];
	char 	buf[17];
	int		i, key, dt, dt_idx;

	// read Date & Time
	rtcReadNowTime((unsigned char *)data);

	// Set from YYYY to ss (YYYY/MM/DD wday hh:mm:ss)
	for(i = 0; i < sizeof(item)/sizeof(item[0]);){

		// display value
		dt_idx = item[i].idx;

		if(dt_idx != ITEM_WDAY){
			mysprintf(buf, item[i].format, bcd2bin(data[dt_idx]));
		}else{
			mysprintf(buf, item[i].format, wday_name[(int)data[dt_idx]]);
		}
		lcd_cmdwrite(LCD_SET_LINE2);
		lcd_strwrite(buf);

		// wait for key 
		while((key = getkey()) == KEY_NONE);

		// change value
		if(key == KEY_UP){
			dt = bcd2bin(data[dt_idx]);
			if(++dt > item[i].max) dt = item[i].min;
			data[dt_idx] = bin2bcd(dt);
		}else if(key == KEY_DOWN){
			dt = bcd2bin(data[dt_idx]);
			if(--dt < item[i].min) dt = item[i].max;
			data[dt_idx] = bin2bcd(dt);
		}else if(key == KEY_SET){
			// display set now time
			display_date_time((unsigned char *)data);
			++i;
		}	
	}

	// set Now Date & Time to RTCC
	rtcWriteTime((unsigned char *)data);

	// display alarm time
	rtcReadAlarmTime((unsigned char *)data);
	display_alarm_time((unsigned char *)data);

}


/*============================================================
	Alarm time Set Operation
============================================================*/

void	alarm_set(void)
{
	char	data[7];
	char 	buf[17];
	int		i, dt, key, dt_idx, item_idx;

	// read Alarm Time
	rtcReadAlarmTime((unsigned char *)data);

	// Set from hh to mm (mm:ss)
	for(i = 0; i < 2;){

		item_idx = i + 4;	// index 4 is Hour value setting.
		dt_idx = item[item_idx].idx -1;	// ITEM_HOUR - 1 = ALARM_HOUR 
		data[dt_idx] &= 0x7f;	// clear AE flag
		
		// display value
		mysprintf(buf, item[item_idx].format, bcd2bin(data[dt_idx]));
		lcd_cmdwrite(LCD_SET_LINE2);
		lcd_strwrite(buf);

		// wait for key 
		while((key = getkey()) == KEY_NONE);

		// change value
		if(key == KEY_UP){
			dt = bcd2bin(data[dt_idx]);
			if(++dt > item[item_idx].max) dt = item[item_idx].min;
			data[dt_idx] = bin2bcd(dt);

		}else if(key == KEY_DOWN){
			dt = bcd2bin(data[dt_idx]);
			if(--dt < item[item_idx].min) dt = item[item_idx].max;
			data[dt_idx] = bin2bcd(dt);

		}else if(key == KEY_SET){
			i++;
		}	
	}


	// set Now Alarm time
	data[ALARM_HOUR] &= 0x3f;
	data[ALARM_MINUTE] &= 0x7f;
	data[ALARM_DAY] = 0x81;
	data[ALARM_WDAY] = 0x80;
	rtcWriteAlarmTime((unsigned char *)data);

	// display alarm time
	display_alarm_time((unsigned char *)data);

}



/*==========================================================
	alarm on
==========================================================*/

void	alarm_on(void)
{
	unsigned char	data[4];

	// already setting
	if(AlarmSet) return;

	// Set Flag is on
	AlarmSet = 1;

	// set alarm interrupt
	rtcSetAlarm();

	// read Alarm Time
	rtcReadAlarmTime(data);

	// display alarm time
	display_alarm_time(data);

	// display Bell icon
	icon_bell(ON);

}


/*==========================================================
	alarm off
==========================================================*/

void	alarm_off(void)
{
	unsigned char	data[4];

	// already no setting
	if(! AlarmSet) return;

	// Set Flag is off
	AlarmSet = 0;

	// clear alarm interrupt
	rtcClearAlarm();

	// read Alarm Time
	rtcReadAlarmTime(data);

	// display alarm time
	display_alarm_time(data);

	// display clear Bell icon
	icon_bell(OFF);

}


/*-----------------------------------------------------------
	LCD Normal Display
	0123456789ABCDEF
	YYYY/MM/DD hh:mm
	       off --:--
	       on  hh:mm
-----------------------------------------------------------*/

void	display_date_time(unsigned char data[])
{
	char	buf[17];

	mysprintf(buf, "%04d/%02d/%02d %02d:%02d",
		(int)bcd2bin(data[ITEM_YEAR])+2000,
		bcd2bin(data[ITEM_MONTH]), 
		bcd2bin(data[ITEM_DAY]),
		bcd2bin(data[ITEM_HOUR]),
		bcd2bin(data[ITEM_MINUTE]));

	lcd_cmdwrite(LCD_SET_LINE1);
	lcd_strwrite(buf);
} 

void	display_alarm_time(unsigned char data[])
{
	char	buf[17];

	if(AlarmSet){
		mysprintf(buf, " alarm on  %02d:%02d", bcd2bin(data[ALARM_HOUR]), bcd2bin(data[ALARM_MINUTE]));
	}else{
		mysprintf(buf, " alarm off --:--");
	}
	lcd_cmdwrite(LCD_SET_LINE2);
	lcd_strwrite(buf);
}


/*** end of main.c ******************************************************************/
